#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from unittest.mock import MagicMock

from source_outbrain_amplify.source import SourceOutbrainAmplify


def test_check_connection(mocker):
    config = {
        "credentials": {"type": "access_token", "access_token": "MTY1OTUyO"},
        "report_granularity": "daily",
        "geo_location_breakdown": "region",
        "start_date": "2022-04-01",
        "end_date": "2022-04-30",
    }
    source = SourceOutbrainAmplify()
    cond = source.check_connection(True, config)[0]
    assert cond is False


def test_streams(mocker):
    source = SourceOutbrainAmplify()
    config_mock = MagicMock()
    streams = source.streams(config_mock)
    expected_streams_number = 17
    assert len(streams) == expected_streams_number



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#


from airbyte_cdk.models import SyncMode
from pytest import fixture
from source_outbrain_amplify.source import IncrementalOutbrainAmplifyStream


@fixture
def patch_incremental_base_class(mocker):
    # Mock abstract methods to enable instantiating abstract class
    mocker.patch.object(IncrementalOutbrainAmplifyStream, "path", "v0/example_endpoint")
    mocker.patch.object(IncrementalOutbrainAmplifyStream, "primary_key", "test_primary_key")
    mocker.patch.object(IncrementalOutbrainAmplifyStream, "__abstractmethods__", set())


def test_cursor_field(patch_incremental_base_class):
    stream = IncrementalOutbrainAmplifyStream()
    # TODO: replace this with your expected cursor field
    expected_cursor_field = []
    assert stream.cursor_field == expected_cursor_field


def test_get_updated_state(patch_incremental_base_class):
    stream = IncrementalOutbrainAmplifyStream()
    # TODO: replace this with your input parameters
    inputs = {"current_stream_state": None, "latest_record": None}
    # TODO: replace this with your expected updated stream state
    expected_state = {}
    assert stream.get_updated_state(**inputs) == expected_state


def test_stream_slices(patch_incremental_base_class):
    stream = IncrementalOutbrainAmplifyStream()
    # TODO: replace this with your input parameters
    inputs = {"sync_mode": SyncMode.incremental, "cursor_field": [], "stream_state": {}}
    # TODO: replace this with your expected stream slices list
    expected_stream_slice = [None]
    assert stream.stream_slices(**inputs) == expected_stream_slice


def test_supports_incremental(patch_incremental_base_class, mocker):
    mocker.patch.object(IncrementalOutbrainAmplifyStream, "cursor_field", "dummy_field")
    stream = IncrementalOutbrainAmplifyStream()
    assert stream.supports_incremental


def test_source_defined_cursor(patch_incremental_base_class):
    stream = IncrementalOutbrainAmplifyStream()
    assert stream.source_defined_cursor


def test_stream_checkpoint_interval(patch_incremental_base_class):
    stream = IncrementalOutbrainAmplifyStream()
    # TODO: replace this with your expected checkpoint interval
    expected_checkpoint_interval = None
    assert stream.state_checkpoint_interval == expected_checkpoint_interval



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

import inspect
import json
from http import HTTPStatus
from unittest.mock import MagicMock

import pytest
import requests
from source_outbrain_amplify.source import OutbrainAmplifyStream


@pytest.fixture
def patch_base_class(mocker):
    mocker.patch.object(OutbrainAmplifyStream, "path", "v0/example_endpoint")
    mocker.patch.object(OutbrainAmplifyStream, "primary_key", "test_primary_key")
    mocker.patch.object(OutbrainAmplifyStream, "__abstractmethods__", set())


def test_request_params(patch_base_class):
    stream = OutbrainAmplifyStream()
    inputs = {"stream_slice": None, "stream_state": None, "next_page_token": None}
    expected_params = {}
    assert stream.request_params(**inputs) == expected_params


def test_next_page_token(patch_base_class):
    stream = OutbrainAmplifyStream()
    inputs = {"totalCount": 100, "count": 50}
    pagination_token = json.dumps(inputs)
    response = requests.Response()
    response.status_code = 200
    response.headers["content-type"] = "application/json"
    response._content = pagination_token.encode("utf-8")
    expected_token = {"offset": 51}
    assert stream.next_page_token(response) == expected_token


def test_parse_response(patch_base_class):
    stream = OutbrainAmplifyStream()
    mock_response = {"campaigns": [], "totalCount": 5, "count": 0}
    mock_response = json.dumps(mock_response)
    response = requests.Response()
    response.status_code = 200
    response.headers["Content-Type"] = "application/json"
    response._content = mock_response.encode("utf-8")
    result = stream.parse_response(response)
    expected_result = True
    assert inspect.isgenerator(result) == expected_result


def test_request_headers(patch_base_class):
    stream = OutbrainAmplifyStream()
    inputs = {"stream_slice": None, "stream_state": None, "next_page_token": None}
    expected_headers = {}
    assert stream.request_headers(**inputs) == expected_headers


def test_http_method(patch_base_class):
    stream = OutbrainAmplifyStream()
    expected_method = "GET"
    assert stream.http_method == expected_method


@pytest.mark.parametrize(
    ("http_status", "should_retry"),
    [
        (HTTPStatus.OK, False),
        (HTTPStatus.BAD_REQUEST, False),
        (HTTPStatus.TOO_MANY_REQUESTS, True),
        (HTTPStatus.INTERNAL_SERVER_ERROR, True),
    ],
)
def test_should_retry(patch_base_class, http_status, should_retry):
    response_mock = MagicMock()
    response_mock.status_code = http_status
    stream = OutbrainAmplifyStream()
    assert stream.should_retry(response_mock) == should_retry


def test_backoff_time(patch_base_class):
    response_mock = MagicMock()
    stream = OutbrainAmplifyStream()
    expected_backoff_time = None
    assert stream.backoff_time(response_mock) == expected_backoff_time


def test_get_time_interval(patch_base_class):
    stream = OutbrainAmplifyStream()
    start_date = "2022-08-03 00:00:00"
    ending_date = "2022-08-02 00:00:00"
    try:
        stream._get_time_interval(start_date, ending_date)
    except ValueError as e:
        assert e
